@@ -4,12 +4,12 @@
 
 /* Package information */
 #define PACKAGE           "task"
-#define VERSION           "2.0.0"
+#define VERSION           "2.1.1"
 #define PACKAGE_BUGREPORT "support@taskwarrior.org"
 #define PACKAGE_NAME      "task"
 #define PACKAGE_TARNAME   "task"
-#define PACKAGE_VERSION   "2.0.0"
-#define PACKAGE_STRING    "task 2.0.0"
+#define PACKAGE_VERSION   "2.1.1"
+#define PACKAGE_STRING    "task 2.1.1"
 
 /* Installation details */
 #define TASK_RCDIR "/usr/local/share/doc/task/rc"
@@ -36,6 +36,7 @@ Override PACKAGE_LANGUAGE, then
 /* #undef CYGWIN */
 /* #undef FREEBSD */
 /* #undef OPENBSD */
+/* #undef NETBSD */
 /* #undef HAIKU */
 /* #undef SOLARIS */
 /* #undef WIN32 */
@@ -57,6 +58,12 @@ Override PACKAGE_LANGUAGE, then
 /* Found srandom */
 #define HAVE_SRANDOM
 
+/* Found tm_gmtoff */
+#define HAVE_TM_GMTOFF
+
+/* Found timegm */
+#define HAVE_TIMEGM
+
 /* Found the uuid library */
 #define HAVE_UUID
 #define HAVE_UUID_UNPARSE_LOWER
