@@ -1,51 +1,57 @@
-///////////////////////////////////////////////////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
+
+#define L10N                                           // Localization complete.
+
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <algorithm>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <inttypes.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <pwd.h>
-#include "Directory.h"
-#include "Date.h"
-#include "File.h"
-#include "Config.h"
-#include "text.h"
-#include "util.h"
-#include "../auto.h"
+#include <Directory.h>
+#include <Date.h>
+#include <File.h>
+#include <Timer.h>
+#include <Config.h>
+#include <text.h>
+#include <util.h>
+#include <cmake.h>
+#include <i18n.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 // This string is used in two ways:
 // 1) It is used to create a new .taskrc file, by copying it directly to disk.
 // 2) It is parsed and used as default values for all Config.get calls.
-std::string Config::defaults =
+std::string Config::_defaults =
   "# Taskwarrior program configuration file.\n"
   "# For more documentation, see http://taskwarrior.org or try 'man task', 'man task-faq',\n"
   "# 'man task-tutorial', 'man task-color', 'man task-sync' or 'man taskrc'\n"
@@ -61,34 +67,50 @@ std::string Config::defaults =
   "data.location=~/.task\n"
   "locking=on                                     # Use file-level locking\n"
   "gc=on                                          # Garbage-collect data files - DO NOT CHANGE unless you are sure\n"
+  "exit.on.missing.db=no                          # Whether to exit if ~/.task is not found\n"
   "\n"
   "# Terminal\n"
-  "curses=on                                      # Detects terminal width\n"
+  "detection=on                                   # Detects terminal width\n"
   "defaultwidth=80                                # Without detection, assumed width\n"
+  "defaultheight=24                               # Without detection, assumed height\n"
+  "avoidlastcolumn=no                             # Fixes Cygwin width problem\n"
+  "hyphenate=on                                   # Hyphenates lines wrapped on non-word-breaks\n"
   "#editor=vi                                     # Preferred text editor\n"
-  "edit.verbose=yes                               # Include comments in files created during task edit\n"
   "\n"
   "# Miscellaneous\n"
-  "verbose=yes                                    # Provide extra feedback\n"
+  "verbose=yes                                    # Provide maximal feedback\n"
+  "#verbose=no                                    # Provide regular feedback\n"
+  "#verbose=nothing                               # Provide no feedback\n"
+  "#                                              # Comma-separated list.  May contain any subset of:\n"
+  "#verbose=blank,header,footnote,label,new-id,affected,edit,special,project\n"
   "confirmation=yes                               # Confirmation on delete, big changes\n"
-  "echo.command=yes                               # Details on command just run\n"
-  "annotations=full                               # Level of verbosity for annotations: full, sparse or none\n"
-  "next=2                                         # How many tasks per project in next report\n"
-  "bulk=2                                         # > 2 tasks considered 'a lot', for confirmation\n"
-  "nag=You have higher priority tasks.            # Nag message to keep you honest\n"                      // TODO
+  "indent.annotation=2                            # Indent spaces for annotations\n"
+  "indent.report=0                                # Indent spaces for whole report\n"
+  "row.padding=0                                  # Left and right padding for each row of report\n"
+  "column.padding=1                               # Spaces between each column in a report\n"
+  "bulk=3                                         # 3 or more tasks considered a bulk change and is confirmed\n"
+  "nag=You have more urgent tasks.                # Nag message to keep you honest\n"                      // TODO
   "search.case.sensitive=yes                      # Setting to no allows case insensitive searches\n"
   "active.indicator=*                             # What to show as an active task indicator\n"
   "tag.indicator=+                                # What to show as a tag indicator\n"
+  "dependency.indicator=D                         # What to show as a dependency indicator\n"
   "recurrence.indicator=R                         # What to show as a task recurrence indicator\n"
   "recurrence.limit=1                             # Number of future recurring pending tasks\n"
   "undo.style=side                                # Undo style - can be 'side', or 'diff'\n"
   "burndown.bias=0.666                            # Weighted mean bias toward recent data\n"
   "regex=no                                       # Assume all search/filter strings are regexes\n"
   "xterm.title=no                                 # Sets xterm title for some commands\n"
+  "expressions=on                                 # Support for algebraic expressions\n"
+  "patterns=on                                    # Support for regex patterns\n"
+  "dom=on                                         # Support DOM access\n"
+  "json.array=off                                 # Enclose JSON output in [ ]\n"
+  "abbreviation.minimum=2                         # Shortest allowed abbreviation\n"
   "\n"
   "# Dates\n"
   "dateformat=m/d/Y                               # Preferred input and display date format\n"
   "dateformat.holiday=YMD                         # Preferred input date format for holidays\n"
+  "dateformat.edit=m/d/Y H:N:S                    # Preferred display date format when editing\n"
+  "dateformat.info=m/d/Y H:N:S                    # Preferred display date format for information\n"
   "dateformat.report=m/d/Y                        # Preferred display date format for reports\n"
   "dateformat.annotation=m/d/Y                    # Preferred display date format for annotations\n"
   "weekstart=Sunday                               # Sunday or Monday only\n"
@@ -115,25 +137,30 @@ std::string Config::defaults =
   "dependency.confirmation=on                     # Should dependency chain repair be confirmed?\n"
   "\n"
   "# Urgency Coefficients\n"
-  "urgency.next.coefficient=10.0                  # Urgency coefficients for 'next' special tag\n"
-  "urgency.blocking.coefficient=9.0               # Urgency coefficients for blocking tasks\n"
-  "urgency.blocked.coefficient=8.0                # Urgency coefficients for blocked tasks\n"
-  "urgency.due.coefficient=7.0                    # Urgency coefficients for due dates\n"
-  "urgency.priority.coefficient=6.0               # Urgency coefficients for priorities\n"
-  "urgency.waiting.coefficient=5.0                # Urgency coefficients for waiting status\n"
-  "urgency.active.coefficient=4.0                 # Urgency coefficients for active tasks\n"
-  "urgency.project.coefficient=3.0                # Urgency coefficients for projects\n"
-  "urgency.tags.coefficient=2.0                   # Urgency coefficients for tags\n"
-  "urgency.annotations.coefficient=1.0            # Urgency coefficients for annotations\n"
+  "urgency.next.coefficient=15.0                  # Urgency coefficient for 'next' special tag\n"
+  "urgency.due.coefficient=12.0                   # Urgency coefficient for due dates\n"
+  "urgency.blocking.coefficient=8.0               # Urgency coefficient for blocking tasks\n"
+  "urgency.priority.coefficient=6.0               # Urgency coefficient for priorities\n"
+  "urgency.active.coefficient=4.0                 # Urgency coefficient for active tasks\n"
+  "urgency.scheduled.coefficient=5.0              # Urgency coefficient for scheduled tasks\n"
+  "urgency.age.coefficient=2.0                    # Urgency coefficient for age\n"
+  "urgency.annotations.coefficient=1.0            # Urgency coefficient for annotations\n"
+  "urgency.tags.coefficient=1.0                   # Urgency coefficient for tags\n"
+  "urgency.project.coefficient=1.0                # Urgency coefficient for projects\n"
+  "urgency.blocked.coefficient=-5.0               # Urgency coefficient for blocked tasks\n"
+  "urgency.waiting.coefficient=-3.0               # Urgency coefficient for waiting status\n"
+  "urgency.age.max=365                            # Maximum age in days\n"
   "\n"
   "#urgency.user.project.foo.coefficient=5.0      # Urgency coefficients for 'foo' project\n"
   "#urgency.user.tag.foo.coefficient=5.0          # Urgency coefficients for 'foo' tag\n"
+  "#urgency.uda.foo.coefficient=5.0               # Urgency coefficients for UDA 'foo'\n"
   "\n"
   "# Color controls.\n"
   "color=on                                       # Enable color\n"
 #ifdef LINUX
   "color.header=color3                            # Color of header messages\n"
   "color.footnote=color3                          # Color of footnote messages\n"
+  "color.error=color3                             # Color of error messages\n"
   "color.debug=color3                             # Color of diagnostic output\n"
   "color.alternate=on color233                    # Alternate color for line coloring\n"
   "\n"
@@ -174,15 +201,20 @@ std::string Config::defaults =
   "#color.tag.bug=yellow                          # Color of +bug tasks\n"
   "#color.tag.none=                               # Color of tag-less tasks\n"
   "color.active=rgb555 on rgb410                  # Color of active tasks\n"
+  "color.scheduled=rgb444 on rgb410               # Color of scheduled tasks\n"
   "color.pri.none=                                # Color of priority:  tasks\n"
   "color.pri.H=rgb255                             # Color of priority:H tasks\n"
   "color.pri.M=rgb250                             # Color of priority:M tasks\n"
   "color.pri.L=rgb245                             # Color of priority:L tasks\n"
   "color.tagged=rgb031                            # Color of tagged tasks\n"
   "color.blocked=white on color8                  # Color of blocked tasks\n"
+  "color.blocking=white on color6                 # Color of blocking tasks\n"
+  "#color.completed=on blue                       # Color of completed tasks\n"
+  "#color.deleted=on blue                         # Color of deleted tasks\n"
 #else
   "color.header=yellow                            # Color of header messages\n"
   "color.footnote=yellow                          # Color of footnote messages\n"
+  "color.error=yellow                             # Color of error messages\n"
   "color.debug=yellow                             # Color of diagnostic output\n"
   "color.alternate=                               # Alternate color for line coloring\n"
   "\n"
@@ -223,18 +255,22 @@ std::string Config::defaults =
   "#color.tag.bug=yellow                          # Color of +bug tasks\n"
   "#color.tag.none=                               # Color of tag-less tasks\n"
   "color.active=black on bright green             # Color of active tasks\n"
+  "color.scheduled=black on green                 # Color of scheduled tasks\n"
   "color.pri.none=                                # Color of priority:  tasks\n"
   "color.pri.H=bold white                         # Color of priority:H tasks\n"
   "color.pri.M=white                              # Color of priority:M tasks\n"
   "color.pri.L=                                   # Color of priority:L tasks\n"
   "color.tagged=green                             # Color of tagged tasks\n"
   "color.blocked=black on white                   # Color of blocked tasks\n"
+  "color.blocking=black on bright white           # Color of blocking tasks\n"
+  "#color.completed=on blue                       # Color of completed tasks\n"
+  "#color.deleted=on blue                         # Color of deleted tasks\n"
 #endif
   "\n"
   "# Here is the rule precedence order, highest to lowest.\n"
   "# Note that these are just the color rule names, without the leading 'color.'\n"
   "#      and any trailing '.value'.\n"
-  "rule.precedence.color=due.today,active,blocked,overdue,due,keyword,project,tag,recurring,pri,tagged\n"
+  "rule.precedence.color=due.today,active,blocking,blocked,overdue,due,scheduled,keyword.,project.,tag.,recurring,pri.,tagged,completed,deleted\n"
   "\n"
   "# Shadow file support\n"
   "#shadow.file=/tmp/shadow.txt                   # Location of shadow file\n"
@@ -244,23 +280,22 @@ std::string Config::defaults =
   "#default.project=foo                           # Default project for 'add' command\n"
   "#default.priority=M                            # Default priority for 'add' command\n"
   "#default.due=eom                               # Default due date for 'add' command\n"
-  "default.command=list                           # When no arguments are specified\n"
+  "default.command=next                           # When no arguments are specified\n"
   "\n"
   "_forcecolor=no                                 # Forces color to be on, even for non TTY output\n"
-  "blanklines=true                                # Use more whitespace in output\n"
   "complete.all.projects=no                       # Include old project names in '_projects' command\n"
   "complete.all.tags=no                           # Include old tag names in '_ags' command\n"
   "list.all.projects=no                           # Include old project names in 'projects' command\n"
   "list.all.tags=no                               # Include old tag names in 'tags' command\n"
   "debug=no                                       # Display diagnostics\n"
-  "hooks=off                                      # Hook system master switch\n"
+  "extensions=off                                 # Extension system master switch\n"
   "fontunderline=yes                              # Uses underlines rather than -------\n"
   "shell.prompt=task>                             # Prompt used by the shell command\n"
   "\n"
 	"# Merge options\n"
   "#\n"
   "# WARNING: Please read the documentation (man task-sync) before proceeding with these\n"
-  "#          synchronization features.  Improperly used, data can be lost!\n"
+  "#          synchronization features.  If improperly used, data can be lost!\n"
 	"merge.autopush=ask                             # Push database to remote origin after merge: yes, no, ask\n"
 	"#merge.default.uri=user@host.xz:.task/         # URI for merge\n"
 	"#pull.default.uri=rsync://host.xz/task-backup/ # URI for pull\n"
@@ -281,159 +316,110 @@ std::string Config::defaults =
   "#import.synonym.tags=?\n"
   "#import.synonym.uuid=?\n"
   "\n"
-  "# Export Controls\n"
-  "export.ical.class=PRIVATE                      # Could be PUBLIC, PRIVATE or CONFIDENTIAL\n"
-  "\n"
   "# Aliases - alternate names for commands\n"
   "alias.rm=delete                                # Alias for the delete command\n"
   "alias.history=history.monthly                  # Prefer monthly over annual history reports\n"
   "alias.ghistory=ghistory.monthly                # Prefer monthly graphical over annual history reports\n"
-  "alias.export=export.yaml                       # Prefer YAML over CSV or iCal export\n"
-  "alias.export.vcalendar=export.ical             # They are the same\n"
-  "alias.export.json=_query                       # The _query command will be used for all export\n"
+  "alias._query=export                            # _query is now export\n"
   "alias.burndown=burndown.weekly                 # Prefer the weekly burndown chart\n"
   "\n"
-  "# Fields: id, uuid, project, priority, priority_long, entry, start, end,\n"
-  "#         due, countdown, countdown_compact, age, age_compact, active, tags,\n"
-  "#         depends, description_only, description, recur, recurrence_indicator,\n"
-  "#         tag_indicator, wait.\n"
-  "# Description:   This report is ...\n"
-  "# Sort:          due+,priority-,project+\n"
-  "# Filter:        pro:x pri:H +bug limit:10\n"
-  "# Dateformat:    due date format in reports\n"
+  "# Reports\n"
   "\n"
-  "# task long\n"
-  "report.long.description=Lists all task, all data, matching the specified criteria\n"
-  "report.long.columns=id,project,priority,entry,start,due,recur,countdown,age,depends,tags,description\n"
+  "report.long.description=Lists all pending tasks\n"
+  "report.long.columns=id,project,priority,entry,start,due,recur,due.countdown,entry.age,depends,tags,description\n"
   "report.long.labels=ID,Project,Pri,Added,Started,Due,Recur,Countdown,Age,Deps,Tags,Description\n"
-  "report.long.sort=due+,priority-,project+,description+\n"
+  "report.long.sort=due+,priority-,project+\n"
   "report.long.filter=status:pending\n"
-  "#report.long.dateformat=m/d/Y\n"
-  "#report.long.annotations=full\n"
   "\n"
-  "# task list\n"
-  "report.list.description=Lists all tasks matching the specified criteria\n"
-  "report.list.columns=id,project,priority,due,active,age,description\n"
+  "report.list.description=Lists all pending tasks\n"
+  "report.list.columns=id,project,priority,due,start.active,entry.age,description\n"
   "report.list.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.list.sort=due+,priority-,active-,project+,description+\n"
+  "report.list.sort=due+,priority-,start-,project+\n"
   "report.list.filter=status:pending\n"
-  "#report.list.dateformat=m/d/Y\n"
-  "#report.list.annotations=full\n"
   "\n"
-  "# task ls\n"
-  "report.ls.description=Minimal listing of all tasks matching the specified criteria\n"
+  "report.ls.description=Minimal listing of all pending tasks\n"
   "report.ls.columns=id,project,priority,description\n"
   "report.ls.labels=ID,Project,Pri,Description\n"
-  "report.ls.sort=priority-,project+,description+\n"
+  "report.ls.sort=priority-,project+\n"
   "report.ls.filter=status:pending\n"
-  "#report.ls.dateformat=m/d/Y\n"
-  "#report.ls.annotations=full\n"
   "\n"
-  "# task minimal\n"
-  "report.minimal.description=A really minimal listing\n"
-  "report.minimal.columns=id,project,description\n"
+  "report.minimal.description=Minimal listing of all pending tasks\n"
+  "report.minimal.columns=id,project,description.truncated\n"
   "report.minimal.labels=ID,Project,Description\n"
   "report.minimal.sort=project+,description+\n"
   "report.minimal.filter=status:pending\n"
-  "#report.minimal.dateformat=m/d/Y\n"
-  "#report.minimal.annotations=full\n"
   "\n"
-  "# task newest\n"
   "report.newest.description=Shows the newest tasks\n"
-  "report.newest.columns=id,project,priority,due,active,age,description\n"
+  "report.newest.columns=id,project,priority,due,start.active,entry.age,description\n"
   "report.newest.labels=ID,Project,Pri,Due,Active,Age,Description\n"
   "report.newest.sort=id-\n"
   "report.newest.filter=status:pending limit:10\n"
-  "#report.newest.dateformat=m/d/Y\n"
-  "#report.newest.annotations=full\n"
   "\n"
-  "# task oldest\n"
   "report.oldest.description=Shows the oldest tasks\n"
-  "report.oldest.columns=id,project,priority,due,active,age,description\n"
+  "report.oldest.columns=id,project,priority,due,start.active,entry.age,description\n"
   "report.oldest.labels=ID,Project,Pri,Due,Active,Age,Description\n"
   "report.oldest.sort=id+\n"
   "report.oldest.filter=status:pending limit:10\n"
-  "#report.oldest.dateformat=m/d/Y\n"
-  "#report.oldest.annotations=full\n"
   "\n"
-  "# task overdue\n"
-  "report.overdue.description=Lists overdue tasks matching the specified criteria\n"
-  "report.overdue.columns=id,project,priority,due,active,age,description\n"
+  "report.overdue.description=Lists overdue tasks\n"
+  "report.overdue.columns=id,project,priority,due,start.active,entry.age,description\n"
   "report.overdue.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.overdue.sort=due+,priority-,active-,project+,description+\n"
+  "report.overdue.sort=due+,priority-,start-,project+\n"
   "report.overdue.filter=status:pending due.before:now\n"
-  "#report.overdue.dateformat=m/d/Y\n"
-  "#report.overdue.annotations=full\n"
   "\n"
-  "# task active\n"
-  "report.active.description=Lists active tasks matching the specified criteria\n"
-  "report.active.columns=id,project,priority,due,active,age,description\n"
+  "report.active.description=Lists active tasks\n"
+  "report.active.columns=id,project,priority,due,start.active,entry.age,description\n"
   "report.active.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.active.sort=due+,priority-,project+,description+\n"
+  "report.active.sort=due+,priority-,project+\n"
   "report.active.filter=status:pending start.any:\n"
-  "#report.active.dateformat=m/d/Y\n"
-  "#report.active.annotations=full\n"
   "\n"
-  "# task completed\n"
-  "report.completed.description=Lists completed tasks matching the specified criteria\n"
-  "report.completed.columns=end,project,priority,age,description\n"
-  "report.completed.labels=Complete,Project,Pri,Age,Description\n"
-  "report.completed.sort=end+,priority-,project+,description+\n"
+  "report.completed.description=Lists completed tasks\n"
+  "report.completed.columns=end,project,priority,entry.age,description,uuid\n"
+  "report.completed.labels=Complete,Project,Pri,Age,Description,UUID\n"
+  "report.completed.sort=end+,priority-,project+\n"
   "report.completed.filter=status:completed\n"
-  "#report.completed.dateformat=m/d/Y\n"
-  "#report.completed.annotations=full\n"
   "\n"
-  "# task recurring\n"
-  "report.recurring.description=Lists recurring tasks matching the specified criteria\n"
-  "report.recurring.columns=id,project,priority,due,recur,active,age,description\n"
+  "report.recurring.description=Lists recurring tasks\n"
+  "report.recurring.columns=id,project,priority,due,recur,start.active,entry.age,description\n"
   "report.recurring.labels=ID,Project,Pri,Due,Recur,Active,Age,Description\n"
-  "report.recurring.sort=due+,priority-,active-,project+,description+\n"
+  "report.recurring.sort=due+,priority-,start-,project+\n"
   "report.recurring.filter=status:pending parent.any:\n"
-  "#report.recurring.dateformat=m/d/Y\n"
-  "#report.recurring.annotations=full\n"
   "\n"
-  "# task waiting\n"
-  "report.waiting.description=Lists all waiting tasks matching the specified criteria\n"
-  "report.waiting.columns=id,project,priority,wait,age,description\n"
+  "report.waiting.description=Lists all waiting tasks\n"
+  "report.waiting.columns=id,project,priority,wait,entry.age,description\n"
   "report.waiting.labels=ID,Project,Pri,Wait,Age,Description\n"
-  "report.waiting.sort=wait+,priority-,project+,description+\n"
+  "report.waiting.sort=wait+,priority-,project+\n"
   "report.waiting.filter=status:waiting\n"
-  "#report.waiting.dateformat=m/d/Y\n"
-  "#report.waiting.annotations=full\n"
   "\n"
-  "# task all\n"
-  "report.all.description=Lists all tasks matching the specified criteria, including parents of recurring tasks\n"
-  "report.all.columns=id,project,priority,due,end,active,age,description\n"
-  "report.all.labels=ID,Project,Pri,Due,Completed,Active,Age,Description\n"
-  "report.all.sort=project+,due+,end+,priority-,active-,description+\n"
+  "report.all.description=Lists all pending and completed tasks\n"
+  "report.all.columns=id,status,project,priority,due,end,start.active,entry.age,description\n"
+  "report.all.labels=ID,Status,Project,Pri,Due,Completed,Active,Age,Description\n"
+  "report.all.sort=entry+\n"
   "report.all.filter=status.not:deleted\n"
-  "#report.all.dateformat=m/d/Y\n"
-  "#report.all.annotations=full\n"
   "\n"
-  "# task next\n"
   "report.next.description=Lists the most urgent tasks\n"
-  "report.next.columns=id,project,priority,due,active,age,description\n"
-  "report.next.labels=ID,Project,Pri,Due,Active,Age,Description\n"
-  "report.next.sort=due+,priority-,active-,project+,description+\n"
-  "report.next.filter=status:pending limit:page depends.none:\n"
-  "#report.next.dateformat=m/d/Y\n"
-  "#report.next.annotations=full\n"
+  "report.next.columns=id,project,priority,due,start.active,entry.age,urgency,description\n"
+  "report.next.filter=status:pending limit:page\n"
+  "report.next.labels=ID,Project,Pri,Due,A,Age,Urgency,Description\n"
+  "report.next.sort=urgency-,due+,priority-,start-,project+\n"
   "\n"
-  "# task blocked\n"
-  "report.blocked.description=Lists all blocked tasks matching the specified criteria\n"
-  "report.blocked.columns=id,depends,project,priority,due,active,age,description\n"
+  "report.ready.description=Lists the most urgent tasks\n"
+  "report.ready.columns=id,project,priority,due,start.active,entry.age,urgency,description\n"
+  "report.ready.filter=status:pending limit:page wait.none: '(scheduled.none: or scheduled.before:now )'\n"
+  "report.ready.labels=ID,Project,Pri,Due,A,Age,Urgency,Description\n"
+  "report.ready.sort=urgency-,due+,priority-,start-,project+\n"
+  "\n"
+  "report.blocked.description=Lists all blocked tasks\n"
+  "report.blocked.columns=id,depends,project,priority,due,start.active,entry.age,description\n"
   "report.blocked.labels=ID,Deps,Project,Pri,Due,Active,Age,Description\n"
-  "report.blocked.sort=due+,priority-,active-,project+,description+\n"
+  "report.blocked.sort=due+,priority-,start-,project+\n"
   "report.blocked.filter=status:pending depends.any:\n"
-  "#report.blocked.dateformat=m/d/Y\n"
   "\n"
-  "# task unblocked\n"
-  "report.unblocked.description=Lists all unblocked tasks matching the specified criteria\n"
-  "report.unblocked.columns=id,depends,project,priority,due,active,age,description\n"
+  "report.unblocked.description=Lists all unblocked tasks\n"
+  "report.unblocked.columns=id,depends,project,priority,due,start.active,entry.age,description\n"
   "report.unblocked.labels=ID,Deps,Project,Pri,Due,Active,Age,Description\n"
-  "report.unblocked.sort=due+,priority-,active-,project+,description+\n"
+  "report.unblocked.sort=due+,priority-,start-,project+\n"
   "report.unblocked.filter=status:pending depends.none:\n"
-  "#report.unblocked.dateformat=m/d/Y\n"
   "\n";
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -445,7 +431,7 @@ std::string Config::defaults =
 //
 // In all real use cases, Config::load is called.
 Config::Config ()
-: original_file ()
+: _original_file ()
 {
 }
 
@@ -466,15 +452,16 @@ Config::Config (const std::string& file)
 //
 void Config::load (const std::string& file, int nest /* = 1 */)
 {
+  Timer timer ("Config::load (" + file + ")");
+
   if (nest > 10)
-    throw std::string ("Configuration file nested to more than 10 levels deep"
-                       " - this has to be a mistake.");
+    throw std::string (STRING_CONFIG_OVERNEST);
 
   // First time in, load the default values.
   if (nest == 1)
   {
     setDefaults ();
-    original_file = File (file);
+    _original_file = File (file);
   }
 
   // Read the file, then parse the contents.
@@ -529,13 +516,13 @@ void Config::parse (const std::string& input, int nest /* = 1 */)
             if (included.readable ())
               this->load (included, nest + 1);
             else
-              throw std::string ("Could not read include file '") + included.data + "'.";
+              throw format (STRING_CONFIG_READ_INCLUDE, included._data);
           }
           else
-            throw std::string ("Can only include files with absolute paths, not '") + included.data + "'";
+            throw format (STRING_CONFIG_INCLUDE_PATH, included._data);
         }
         else
-          throw std::string ("Malformed entry '") + line + "'.";
+          throw format (STRING_CONFIG_BAD_ENTRY, line);
       }
     }
   }
@@ -545,7 +532,7 @@ void Config::parse (const std::string& input, int nest /* = 1 */)
 void Config::createDefaultRC (const std::string& rc, const std::string& data)
 {
   // Override data.location in the defaults.
-  std::string::size_type loc = defaults.find ("data.location=~/.task");
+  std::string::size_type loc = _defaults.find ("data.location=~/.task");
   //                                      loc+0^          +14^   +21^
 
   Date now;
@@ -555,23 +542,24 @@ void Config::createDefaultRC (const std::string& rc, const std::string& data)
            << " "
            << now.toString ("m/d/Y H:N:S")
            << "]\n"
-           << defaults.substr (0, loc + 14)
+           << _defaults.substr (0, loc + 14)
            << data
            << "\n\n# Color theme (uncomment one to use)\n"
-           << "#include /usr/local/share/doc/task/rc/light-16.theme\n"
-           << "#include /usr/local/share/doc/task/rc/light-256.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-16.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-256.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-red-256.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-green-256.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-blue-256.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-violets-256.theme\n"
-           << "#include /usr/local/share/doc/task/rc/dark-yellow-green.theme\n"
+           << "#include " << TASK_RCDIR << "/light-16.theme\n"
+           << "#include " << TASK_RCDIR << "/light-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-16.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-red-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-green-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-blue-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-violets-256.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-yellow-green.theme\n"
+           << "#include " << TASK_RCDIR << "/dark-gray-256.theme\n"
            << "\n";
 
   // Write out the new file.
   if (! File::write (rc, contents.str ()))
-    throw std::string ("Could not write to '") + rc + "'.";
+    throw format (STRING_CONFIG_BAD_WRITE, rc);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -579,13 +567,18 @@ void Config::createDefaultData (const std::string& data)
 {
   Directory d (data);
   if (! d.exists ())
+  {
+    if (getBoolean ("exit.on.missing.db"))
+      throw std::string ("Error: rc.data.location does not exist - exiting according to rc.exit.on.missing.db setting.");
+
     d.create ();
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Config::setDefaults ()
 {
-  parse (defaults);
+  parse (_defaults);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -605,7 +598,7 @@ const std::string Config::get (const std::string& key)
 const int Config::getInteger (const std::string& key)
 {
   if ((*this).find (key) != (*this).end ())
-    return atoi ((*this)[key].c_str ());
+    return strtoimax ((*this)[key].c_str (), NULL, 10);
 
   return 0;
 }
@@ -614,7 +607,7 @@ const int Config::getInteger (const std::string& key)
 const double Config::getReal (const std::string& key)
 {
   if ((*this).find (key) != (*this).end ())
-    return atof ((*this)[key].c_str ());
+    return strtod ((*this)[key].c_str (), NULL);
 
   return 0.0;
 }
@@ -625,15 +618,15 @@ const bool Config::getBoolean (const std::string& key)
   if ((*this).find (key) != (*this).end ())
   {
     std::string value = lowerCase ((*this)[key]);
-    if (value == "t"      ||  // TODO i18n
-        value == "true"   ||  // TODO i18n
-        value == "1"      ||  // no i18n
-        value == "+"      ||  // no i18n
-        value == "y"      ||  // TODO i18n
-        value == "yes"    ||  // TODO i18n
-        value == "on"     ||  // TODO i18n
-        value == "enable" ||  // TODO i18n
-        value == "enabled")   // TODO i18n
+    if (value == "t"      ||  // TODO Deprecate
+        value == "true"   ||
+        value == "1"      ||
+        value == "+"      ||  // TODO Deprecate
+        value == "y"      ||
+        value == "yes"    ||
+        value == "on"     ||
+        value == "enable" ||  // TODO Deprecate
+        value == "enabled")   // TODO Deprecate
       return true;
   }
 
@@ -668,69 +661,3 @@ void Config::all (std::vector<std::string>& items) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-std::string Config::checkForDeprecatedColor ()
-{
-  std::vector <std::string> deprecated;
-  std::map <std::string, std::string>::const_iterator it;
-  for (it = this->begin (); it != this->end (); ++it)
-  {
-    if (it->first.find ("color.") != std::string::npos)
-    {
-      std::string value = get (it->first);
-      if (value.find ("_") != std::string::npos)
-        deprecated.push_back (it->first);
-    }
-  }
-
-  std::stringstream out;
-  if (deprecated.size ())
-  {
-    out << "Your .taskrc file contains color settings that use deprecated "
-        << "underscores.  Please check:\n";
-
-    std::vector <std::string>::iterator it2;
-    for (it2 = deprecated.begin (); it2 != deprecated.end (); ++it2)
-      out << "  " << *it2 << "=" << get (*it2) << "\n";
-
-    out << "\n";
-  }
-
-  return out.str ();
-}
-
-////////////////////////////////////////////////////////////////////////////////
-std::string Config::checkForDeprecatedColumns ()
-{
-  std::vector <std::string> deprecated;
-  std::map <std::string, std::string>::const_iterator it;
-  for (it = this->begin (); it != this->end (); ++it)
-  {
-    if (it->first.find ("report") == 0)
-    {
-      std::string value = get (it->first);
-      if (value.find ("entry_time") != std::string::npos ||
-          value.find ("start_time") != std::string::npos ||
-          value.find ("end_time")   != std::string::npos)
-        deprecated.push_back (it->first);
-    }
-  }
-
-  std::stringstream out;
-  out << "\n";
-
-  if (deprecated.size ())
-  {
-    out << "Your .taskrc file contains reports with deprecated columns.  "
-        << "Please check for entry_time, start_time or end_time in:\n";
-
-    std::vector <std::string>::iterator it2;
-    for (it2 = deprecated.begin (); it2 != deprecated.end (); ++it2)
-      out << "  " << *it2 << "=" << get (*it2) << "\n";
-
-    out << "\n";
-  }
-
-  return out.str ();
-}
-
-////////////////////////////////////////////////////////////////////////////////
