@@ -31,6 +31,7 @@
 #include <fstream>
 #include <sstream>
 #include <algorithm>
+#include <assert.h>
 #include <pwd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -66,6 +67,13 @@ Context::Context ()
 ////////////////////////////////////////////////////////////////////////////////
 Context::~Context ()
 {
+  std::map<std::string, Command*>::iterator com;
+  for (com = commands.begin (); com != commands.end (); ++com)
+    delete com->second;
+
+  std::map<std::string, Column*>::iterator col;
+  for (col = columns.begin (); col != columns.end (); ++col)
+    delete col->second;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -96,7 +104,6 @@ int Context::initialize (int argc, const char** argv)
     if (override)
     {
       rc_file = File (override);
-      debug ("TASKRC override: ");
       header (format (STRING_CONTEXT_RC_OVERRIDE, rc_file._data));
     }
 
@@ -124,6 +131,7 @@ int Context::initialize (int argc, const char** argv)
 */
 
     // Create missing config file and data directory, if necessary.
+    a3.apply_overrides ();
     createDefaultConfig ();
 
     // Handle Aliases.
@@ -133,6 +141,10 @@ int Context::initialize (int argc, const char** argv)
     // Apply rc overrides to Context::config, capturing raw args for later use.
     a3.apply_overrides ();
 
+    // Now that the final RC is in place, initialize the urgency coefficients
+    // to speed the 'next' report.
+    initializeUrgencyCoefficients ();
+
     // Initialize the color rules, if necessary.
     if (color ())
       initializeColorRules ();
@@ -197,9 +209,9 @@ int Context::initialize (int argc, const char** argv)
     std::vector <std::string>::iterator d;
     for (d = debugMessages.begin (); d != debugMessages.end (); ++d)
       if (color ())
-        std::cout << colorizeDebug (*d) << "\n";
+        std::cerr << colorizeDebug (*d) << "\n";
       else
-        std::cout << *d << "\n";
+        std::cerr << *d << "\n";
   }
 
   // Dump all headers, controlled by 'header' verbosity token.
@@ -208,9 +220,9 @@ int Context::initialize (int argc, const char** argv)
     std::vector <std::string>::iterator h;
     for (h = headers.begin (); h != headers.end (); ++h)
       if (color ())
-        std::cout << colorizeHeader (*h) << "\n";
+        std::cerr << colorizeHeader (*h) << "\n";
       else
-        std::cout << *h << "\n";
+        std::cerr << *h << "\n";
   }
 
   // Dump all footnotes, controlled by 'footnote' verbosity token.
@@ -219,9 +231,9 @@ int Context::initialize (int argc, const char** argv)
     std::vector <std::string>::iterator f;
     for (f = footnotes.begin (); f != footnotes.end (); ++f)
       if (color ())
-        std::cout << colorizeFootnote (*f) << "\n";
+        std::cerr << colorizeFootnote (*f) << "\n";
       else
-        std::cout << *f << "\n";
+        std::cerr << *f << "\n";
   }
 
   // Dump all errors, non-maskable.
@@ -231,9 +243,9 @@ int Context::initialize (int argc, const char** argv)
     std::vector <std::string>::iterator e;
     for (e = errors.begin (); e != errors.end (); ++e)
       if (color ())
-        std::cout << colorizeFootnote (*e) << "\n";
+        std::cerr << colorizeFootnote (*e) << "\n";
       else
-        std::cout << *e << "\n";
+        std::cerr << *e << "\n";
   }
 
   timer_init.stop ();
@@ -300,9 +312,9 @@ int Context::run ()
     std::vector <std::string>::iterator d;
     for (d = debugMessages.begin (); d != debugMessages.end (); ++d)
       if (color ())
-        std::cout << colorizeDebug (*d) << "\n";
+        std::cerr << colorizeDebug (*d) << "\n";
       else
-        std::cout << *d << "\n";
+        std::cerr << *d << "\n";
   }
 
   // Dump all headers, controlled by 'header' verbosity token.
@@ -311,9 +323,9 @@ int Context::run ()
     std::vector <std::string>::iterator h;
     for (h = headers.begin (); h != headers.end (); ++h)
       if (color ())
-        std::cout << colorizeHeader (*h) << "\n";
+        std::cerr << colorizeHeader (*h) << "\n";
       else
-        std::cout << *h << "\n";
+        std::cerr << *h << "\n";
   }
 
   // Dump the report output.
@@ -325,9 +337,9 @@ int Context::run ()
     std::vector <std::string>::iterator f;
     for (f = footnotes.begin (); f != footnotes.end (); ++f)
       if (color ())
-        std::cout << colorizeFootnote (*f) << "\n";
+        std::cerr << colorizeFootnote (*f) << "\n";
       else
-        std::cout << *f << "\n";
+        std::cerr << *f << "\n";
   }
 
   // Dump all errors, non-maskable.
@@ -335,9 +347,9 @@ int Context::run ()
   std::vector <std::string>::iterator e;
   for (e = errors.begin (); e != errors.end (); ++e)
     if (color ())
-      std::cout << colorizeFootnote (*e) << "\n";
+      std::cerr << colorizeError (*e) << "\n";
     else
-      std::cout << *e << "\n";
+      std::cerr << *e << "\n";
 
   hooks.trigger ("on-exit");
   return rc;
@@ -354,6 +366,8 @@ int Context::dispatch (std::string &out)
   {
     updateXtermTitle ();
 
+    updateVerbosity ();
+
     Command* c = commands[command];
 
     // GC is invoked prior to running any command that displays task IDs, if
@@ -383,7 +397,10 @@ int Context::dispatch (std::string &out)
     return rc;
   }
 
+  assert (commands["help"]);
   return commands["help"]->execute (out);
+
+  return 1;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -459,13 +476,9 @@ bool Context::verbose (const std::string& token)
 
       // This list emulates rc.verbose=off in version 1.9.4.
       verbosity.push_back ("blank");
-      verbosity.push_back ("footnote");
       verbosity.push_back ("label");
       verbosity.push_back ("new-id");
-      verbosity.push_back ("affected");
       verbosity.push_back ("edit");
-      verbosity.push_back ("special");
-      verbosity.push_back ("project");
     }
   }
 
@@ -564,6 +577,10 @@ const std::vector <std::string> Context::getCommands () const
 ////////////////////////////////////////////////////////////////////////////////
 void Context::assumeLocations ()
 {
+  // Note that this pointer is deliberately not free()'d, even though valgrind
+  // complains about it.  It is either not necessary, or forbidden, depending
+  // on OS.
+
   // Set up default locations.
   struct passwd* pw = getpwuid (getuid ());
   if (!pw)
@@ -580,7 +597,8 @@ void Context::createDefaultConfig ()
   // Do we need to create a default rc?
   if (! rc_file.exists ())
   {
-    if (!confirm (format (STRING_CONTEXT_CREATE_RC, home_dir, rc_file._data)))
+    if (config.getBoolean ("confirmation") &&
+        !confirm (format (STRING_CONTEXT_CREATE_RC, home_dir, rc_file._data)))
       throw std::string (STRING_CONTEXT_NEED_RC);
 
     config.createDefaultRC (rc_file, data_dir);
@@ -677,31 +695,50 @@ void Context::updateXtermTitle ()
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// This function allows a clean output if the command is a helper subcommand.
+void Context::updateVerbosity ()
+{
+  std::string command;
+  a3.find_command (command);
+
+  if (command[0] == '_')
+  {
+    verbosity.clear ();
+    verbosity.push_back ("nothing");
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void Context::header (const std::string& input)
 {
-  if (input.length ())
+  if (input.length () &&
+      std::find (headers.begin (), headers.end (), input) == headers.end ())
     headers.push_back (input);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Context::footnote (const std::string& input)
 {
-  if (input.length ())
+  if (input.length () &&
+      std::find (footnotes.begin (), footnotes.end (), input) == footnotes.end ())
     footnotes.push_back (input);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Context::error (const std::string& input)
 {
-  if (input.length ())
+  if (input.length () &&
+      std::find (errors.begin (), errors.end (), input) == errors.end ())
     errors.push_back (input);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Context::debug (const std::string& input)
 {
-  debugMessages.push_back (input);
+  if (input.length () &&
+      std::find (debugMessages.begin (), debugMessages.end (), input) == debugMessages.end ())
+    debugMessages.push_back (input);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
