@@ -1,39 +1,83 @@
 ////////////////////////////////////////////////////////////////////////////////
 // taskwarrior - a command line task list manager.
 //
-// Copyright 2006 - 2011, Paul Beckingham, Federico Hernandez.
-// All rights reserved.
+// Copyright 2006-2012, Paul Beckingham, Federico Hernandez.
 //
-// This program is free software; you can redistribute it and/or modify it under
-// the terms of the GNU General Public License as published by the Free Software
-// Foundation; either version 2 of the License, or (at your option) any later
-// version.
+// Permission is hereby granted, free of charge, to any person obtaining a copy
+// of this software and associated documentation files (the "Software"), to deal
+// in the Software without restriction, including without limitation the rights
+// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+// copies of the Software, and to permit persons to whom the Software is
+// furnished to do so, subject to the following conditions:
 //
-// This program is distributed in the hope that it will be useful, but WITHOUT
-// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-// details.
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
 //
-// You should have received a copy of the GNU General Public License along with
-// this program; if not, write to the
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+// SOFTWARE.
 //
-//     Free Software Foundation, Inc.,
-//     51 Franklin Street, Fifth Floor,
-//     Boston, MA
-//     02110-1301
-//     USA
+// http://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
+
+#define L10N                                           // Localization complete.
+
 #include <iomanip>
 #include <sstream>
 #include <time.h>
 #include <assert.h>
+#include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
-#include "Date.h"
-#include "text.h"
-#include "util.h"
-#include "Context.h"
+#include <Nibbler.h>
+#include <Date.h>
+#include <text.h>
+#include <util.h>
+#include <i18n.h>
+#include <Context.h>
+
+static const char* relatives[] =
+{
+  STRING_DATE_SUNDAY_LONG,
+  STRING_DATE_MONDAY_LONG,
+  STRING_DATE_TUESDAY_LONG,
+  STRING_DATE_WEDNESDAY_LONG,
+  STRING_DATE_THURSDAY_LONG,
+  STRING_DATE_FRIDAY_LONG,
+  STRING_DATE_SATURDAY_LONG,
+  "today",
+  "tomorrow",
+  "yesterday",
+  "eow",
+  "eoww",
+  "eocw",
+  "eom",
+  "eoq",
+  "eoy",
+  "sow",
+  "soww",
+  "socw",
+  "som",
+  "soq",
+  "soy",
+  "goodfriday",
+  "easter",
+  "eastermonday",
+  "ascension",
+  "pentecost",
+  "midsommar",
+  "midsommarafton",
+  "now",
+  "later",
+  "someday",
+};
+
+#define NUM_RELATIVES   (sizeof (relatives)   / sizeof (relatives[0]))
 
 extern Context context;
 
@@ -41,13 +85,13 @@ extern Context context;
 // Defaults to "now".
 Date::Date ()
 {
-  mT = time (NULL);
+  _t = time (NULL);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Date::Date (const time_t t)
 {
-  mT = t;
+  _t = t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -60,7 +104,7 @@ Date::Date (const int m, const int d, const int y)
   t.tm_mon   = m - 1;
   t.tm_year  = y - 1900;
 
-  mT = mktime (&t);
+  _t = mktime (&t);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -77,295 +121,36 @@ Date::Date (const int m,  const int d,  const int y,
   t.tm_min   = mi;
   t.tm_sec   = se;
 
-  mT = mktime (&t);
+  _t = mktime (&t);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Date::Date (const std::string& input, const std::string& format /* = "m/d/Y" */)
 {
-  int month  = 0;
-  int day    = 0;
-  int year   = -1;   // So we can check later.
-  int hour   = 0;
-  int minute = 0;
-  int second = 0;
+  // Before parsing according to "format", perhaps this is a relative date?
+  if (isRelativeDate (input))
+    return;
+
+  // Parse a formatted date.
+  Nibbler n (input);
+  if (n.getDate (format, _t) && n.depleted ())
+    return;
+
+  // Parse an ISO date.
+  if (n.getDateISO (_t) && n.depleted ())
+    return;
 
   // Perhaps it is an epoch date, in string form?
   if (isEpoch (input))
     return;
 
-  // Before parsing according to "format", perhaps this is a relative date?
-  if (isRelativeDate (input))
-    return;
-
-  unsigned int i = 0; // Index into input.
-
-  // Format may include: mMdDyYVaAbBhHNS
-  //
-  // Note that the format should never include T or Z, as that interferes with
-  // the potential parsing for ISO dates constructed from the above format.
-  for (unsigned int f = 0; f < format.length (); ++f)
-  {
-    switch (format[f])
-    {
-    // Single or double digit.
-    case 'm':
-      if (i >= input.length () ||
-          ! isdigit (input[i]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (m).  Expected format '" + format + "'";
-      }
-
-      if (i + 1 < input.length ()                    &&
-          (input[i + 0] == '0' || input[i + 0] == '1') &&
-          isdigit (input[i + 1]))
-      {
-        month = atoi (input.substr (i, 2).c_str ());
-        i += 2;
-      }
-      else
-      {
-        month = atoi (input.substr (i, 1).c_str ());
-        ++i;
-      }
-      break;
-
-    case 'd':
-      if (i >= input.length () ||
-          ! isdigit (input[i]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (d).  Expected format '" + format + "'";
-      }
-
-      if (i + 1 < input.length ()                                                                    &&
-          (input[i + 0] == '0' || input[i + 0] == '1' || input[i + 0] == '2' || input[i + 0] == '3') &&
-          isdigit (input[i + 1]))
-      {
-        day = atoi (input.substr (i, 2).c_str ());
-        i += 2;
-      }
-      else
-      {
-        day = atoi (input.substr (i, 1).c_str ());
-        ++i;
-      }
-      break;
-
-    // Double digit.
-    case 'y':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (y).  Expected format '" + format + "'";
-      }
-
-      year = atoi (input.substr (i, 2).c_str ()) + 2000;
-      i += 2;
-      break;
-
-    case 'M':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (M).  Expected format '" + format + "'";
-      }
-
-      month = atoi (input.substr (i, 2).c_str ());
-      i += 2;
-      break;
-
-    case 'D':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (D).  Expected format '" + format + "'";
-      }
-
-      day = atoi (input.substr (i, 2).c_str ());
-      i += 2;
-      break;
-
-    case 'V':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (V).  Expected format '" + format + "'";
-      }
-
-      i += 2;
-      break;
-
-    // Quadruple digit.
-    case 'Y':
-      if (i + 3 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]) ||
-          ! isdigit (input[i + 2]) ||
-          ! isdigit (input[i + 3]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (Y).  Expected format '" + format + "'";
-      }
-
-      year = atoi (input.substr (i, 4).c_str ());
-      i += 4;
-      break;
-
-    // Short names with 3 characters
-    case 'a':
-      if (i + 2 >= input.length () ||
-          isdigit (input[i + 0]) ||
-          isdigit (input[i + 1]) ||
-          isdigit (input[i + 2]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (a).  Expected format '" + format + "'";
-      }
-
-      i += 3;
-      break;
-
-    case 'b':
-      if (i + 2 >= input.length () ||
-          isdigit (input[i + 0]) ||
-          isdigit (input[i + 1]) ||
-          isdigit (input[i + 2]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (b).  Expected format '" + format + "'";
-      }
-
-      month = Date::monthOfYear (input.substr (i, 3).c_str());
-      i += 3;
-      break;
-
-    // Long names
-    case 'A':
-      if (i + 2 >= input.length () ||
-          isdigit (input[i + 0]) ||
-          isdigit (input[i + 1]) ||
-          isdigit (input[i + 2]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (A).  Expected format '" + format + "'";
-      }
-
-      i += Date::dayName( Date::dayOfWeek (input.substr (i, 3).c_str()) ).size();
-      break;
-
-    case 'B':
-      if (i + 2 >= input.length () ||
-          isdigit (input[i + 0]) ||
-          isdigit (input[i + 1]) ||
-          isdigit (input[i + 2]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (B).  Expected format '" + format + "'";
-      }
-
-      month = Date::monthOfYear (input.substr (i, 3).c_str());
-      i += Date::monthName(month).size();
-      break;
-
-    // Single or double digit.
-    case 'h':
-      if (i >= input.length () ||
-          ! isdigit (input[i]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (h).  Expected format '" + format + "'";
-      }
-
-      if (i + 1 < input.length ()                    &&
-          (input[i + 0] == '0' || input[i + 0] == '1' || input[i + 0] == '2') &&
-          isdigit (input[i + 1]))
-      {
-        hour = atoi (input.substr (i, 2).c_str ());
-        i += 2;
-      }
-      else
-      {
-        hour = atoi (input.substr (i, 1).c_str ());
-        ++i;
-      }
-      break;
-
-    case 'H':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (H).  Expected format '" + format + "'";
-      }
-
-      hour = atoi (input.substr (i, 2).c_str ());
-      i += 2;
-      break;
-
-    case 'N':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (N).  Expected format '" + format + "'";
-      }
-
-      minute = atoi (input.substr (i, 2).c_str ());
-      i += 2;
-      break;
-
-    case 'S':
-      if (i + 1 >= input.length () ||
-          ! isdigit (input[i + 0]) ||
-          ! isdigit (input[i + 1]))
-      {
-        throw std::string ("\"") + input + "\" is not a valid date (S).  Expected format '" + format + "'";
-      }
-
-      second = atoi (input.substr (i, 2).c_str ());
-      i += 2;
-      break;
-
-    default:
-      if (i >= input.length () ||
-          input[i] != format[f])
-      {
-        throw std::string ("\"") + input + "\" is not a valid date.  Expected format '" + format + "'";
-      }
-      ++i;
-      break;
-    }
-  }
-
-  // Default the year to the current year, for formats that lack Y/y.
-  if (year == -1)
-  {
-    time_t now = time (NULL);
-    struct tm* default_year = localtime (&now);
-    year = default_year->tm_year + 1900;
-  }
-
-  if (i < input.length ())
-    throw std::string ("\"") + input + "\" is not a valid date.  Expected format '" + format + "'";
-
-  if (!valid (month, day, year))
-    throw std::string ("\"") + input + "\" is not a valid date but is in a valid format.";
-
-  // Convert to epoch.
-  struct tm t = {0};
-  t.tm_isdst = -1;   // Requests that mktime determine summer time effect.
-  t.tm_mday  = day;
-  t.tm_mon   = month - 1;
-  t.tm_year  = year - 1900;
-  t.tm_hour  = hour;
-  t.tm_min   = minute;
-  t.tm_sec   = second;
-
-  mT = mktime (&t);
+  throw ::format (STRING_DATE_INVALID_FORMAT, input, format);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Date::Date (const Date& rhs)
 {
-  mT = rhs.mT;
+  _t = rhs._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -376,14 +161,14 @@ Date::~Date ()
 ////////////////////////////////////////////////////////////////////////////////
 time_t Date::toEpoch ()
 {
-  return mT;
+  return _t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 std::string Date::toEpochString ()
 {
   std::stringstream epoch;
-  epoch << mT;
+  epoch << _t;
   return epoch.str ();
 }
 
@@ -391,7 +176,7 @@ std::string Date::toEpochString ()
 // 19980119T070000Z =  YYYYMMDDThhmmssZ
 std::string Date::toISO ()
 {
-  struct tm* t = gmtime (&mT);
+  struct tm* t = gmtime (&_t);
 
   std::stringstream iso;
   iso << std::setw (4) << std::setfill ('0') << t->tm_year + 1900
@@ -406,16 +191,22 @@ std::string Date::toISO ()
   return iso.str ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+double Date::toJulian ()
+{
+  return (_t / 86400.0) + 2440587.5;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 void Date::toEpoch (time_t& epoch)
 {
-  epoch = mT;
+  epoch = _t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 void Date::toMDY (int& m, int& d, int& y)
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
 
   m = t->tm_mon + 1;
   d = t->tm_mday;
@@ -423,17 +214,18 @@ void Date::toMDY (int& m, int& d, int& y)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::string Date::toString (const std::string& format /*= "m/d/Y" */) const
+const std::string Date::toString (
+  const std::string& format /*= "m/d/Y" */) const
 {
-  // Making this local copy seems to fix a bug.  Remove the local copy and you'll
-  // see segmentation faults and all kinds of gibberish.
+  // Making this local copy seems to fix a bug.  Remove the local copy and
+  // you'll see segmentation faults and all kinds of gibberish.
   std::string localFormat = format;
 
   char buffer[12];
   std::string formatted;
   for (unsigned int i = 0; i < localFormat.length (); ++i)
   {
-    char c = localFormat[i];
+    int c = localFormat[i];
     switch (c)
     {
     case 'm': sprintf (buffer, "%d",   this->month ());                        break;
@@ -451,6 +243,8 @@ const std::string Date::toString (const std::string& format /*= "m/d/Y" */) cons
     case 'H': sprintf (buffer, "%02d", this->hour ());                         break;
     case 'N': sprintf (buffer, "%02d", this->minute ());                       break;
     case 'S': sprintf (buffer, "%02d", this->second ());                       break;
+    case 'j': sprintf (buffer, "%d",   this->dayOfYear ());                    break;
+    case 'J': sprintf (buffer, "%03d", this->dayOfYear ());                    break;
     default:  sprintf (buffer, "%c",   c);                                     break;
     }
 
@@ -469,7 +263,7 @@ Date Date::startOfDay () const
 ////////////////////////////////////////////////////////////////////////////////
 Date Date::startOfWeek () const
 {
-  Date sow (mT);
+  Date sow (_t);
   sow -= (dayOfWeek () * 86400);
   return Date (sow.month (), sow.day (), sow.year (), 0, 0, 0);
 }
@@ -537,6 +331,20 @@ bool Date::valid (const int m, const int d, const int y)
   return true;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Julian
+bool Date::valid (const int d, const int y)
+{
+  // Check that the year is valid.
+  if (y < 0)
+    return false;
+
+  if (d < 0 || d > 365)
+    return false;
+
+  return true;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool Date::leapYear (int year)
 {
@@ -568,23 +376,23 @@ std::string Date::monthName (int month)
 {
   static const char* months[12] =
   {
-    "January",
-    "February",
-    "March",
-    "April",
-    "May",
-    "June",
-    "July",
-    "August",
-    "September",
-    "October",
-    "November",
-    "December",
+    STRING_DATE_JANUARY_LONG,
+    STRING_DATE_FEBRUARY_LONG,
+    STRING_DATE_MARCH_LONG,
+    STRING_DATE_APRIL_LONG,
+    STRING_DATE_MAY_LONG,
+    STRING_DATE_JUNE_LONG,
+    STRING_DATE_JULY_LONG,
+    STRING_DATE_AUGUST_LONG,
+    STRING_DATE_SEPTEMBER_LONG,
+    STRING_DATE_OCTOBER_LONG,
+    STRING_DATE_NOVEMBER_LONG,
+    STRING_DATE_DECEMBER_LONG,
   };
 
   assert (month > 0);
   assert (month <= 12);
-  return months[month - 1];
+  return ucFirst (months[month - 1]);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -592,16 +400,16 @@ void Date::dayName (int dow, std::string& name)
 {
   static const char* days[7] =
   {
-    "Sunday",
-    "Monday",
-    "Tuesday",
-    "Wednesday",
-    "Thursday",
-    "Friday",
-    "Saturday",
+    STRING_DATE_SUNDAY_LONG,
+    STRING_DATE_MONDAY_LONG,
+    STRING_DATE_TUESDAY_LONG,
+    STRING_DATE_WEDNESDAY_LONG,
+    STRING_DATE_THURSDAY_LONG,
+    STRING_DATE_FRIDAY_LONG,
+    STRING_DATE_SATURDAY_LONG,
   };
 
-  name = days[dow];
+  name = ucFirst (days[dow]);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -609,22 +417,22 @@ std::string Date::dayName (int dow)
 {
   static const char* days[7] =
   {
-    "Sunday",
-    "Monday",
-    "Tuesday",
-    "Wednesday",
-    "Thursday",
-    "Friday",
-    "Saturday",
+    STRING_DATE_SUNDAY_LONG,
+    STRING_DATE_MONDAY_LONG,
+    STRING_DATE_TUESDAY_LONG,
+    STRING_DATE_WEDNESDAY_LONG,
+    STRING_DATE_THURSDAY_LONG,
+    STRING_DATE_FRIDAY_LONG,
+    STRING_DATE_SATURDAY_LONG,
   };
 
-  return days[dow];
+  return ucFirst (days[dow]);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Date::weekOfYear (int weekStart) const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   char   weekStr[3];
 
   if (weekStart == 0)
@@ -632,8 +440,7 @@ int Date::weekOfYear (int weekStart) const
   else if (weekStart == 1)
     strftime(weekStr, sizeof(weekStr), "%V", t);
   else
-    throw std::string ("The 'weekstart' configuration variable may "
-                       "only contain 'Sunday' or 'Monday'.");
+    throw std::string (STRING_DATE_BAD_WEEKSTART);
 
   int weekNumber = atoi (weekStr);
 
@@ -646,7 +453,7 @@ int Date::weekOfYear (int weekStart) const
 ////////////////////////////////////////////////////////////////////////////////
 int Date::dayOfWeek () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_wday;
 }
 
@@ -655,38 +462,77 @@ int Date::dayOfWeek (const std::string& input)
 {
   std::string in = lowerCase (input);
 
-  if (in == "sunday"    || in == "sun")   return 0;
-  if (in == "monday"    || in == "mon")   return 1;
-  if (in == "tuesday"   || in == "tue")   return 2;
-  if (in == "wednesday" || in == "wed")   return 3;
-  if (in == "thursday"  || in == "thu")   return 4;
-  if (in == "friday"    || in == "fri")   return 5;
-  if (in == "saturday"  || in == "sat")   return 6;
+  if (in == STRING_DATE_SUNDAY_LONG    || in == STRING_DATE_SUNDAY_SHORT)    return 0;
+  if (in == STRING_DATE_MONDAY_LONG    || in == STRING_DATE_MONDAY_SHORT)    return 1;
+  if (in == STRING_DATE_TUESDAY_LONG   || in == STRING_DATE_TUESDAY_SHORT)   return 2;
+  if (in == STRING_DATE_WEDNESDAY_LONG || in == STRING_DATE_WEDNESDAY_SHORT) return 3;
+  if (in == STRING_DATE_THURSDAY_LONG  || in == STRING_DATE_THURSDAY_SHORT)  return 4;
+  if (in == STRING_DATE_FRIDAY_LONG    || in == STRING_DATE_FRIDAY_SHORT)    return 5;
+  if (in == STRING_DATE_SATURDAY_LONG  || in == STRING_DATE_SATURDAY_SHORT)  return 6;
 
   return -1;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+int Date::dayOfYear () const
+{
+  struct tm* t = localtime (&_t);
+  return t->tm_yday + 1;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 int Date::monthOfYear (const std::string& input)
 {
   std::string in = lowerCase (input);
 
-  if (in == "january"   || in == "jan")     return  1;
-  if (in == "february"  || in == "feb")     return  2;
-  if (in == "march"     || in == "mar")     return  3;
-  if (in == "april"     || in == "apr")     return  4;
-  if (in == "may"       || in == "may")     return  5;
-  if (in == "june"      || in == "jun")     return  6;
-  if (in == "july"      || in == "jul")     return  7;
-  if (in == "august"    || in == "aug")     return  8;
-  if (in == "september" || in == "sep")     return  9;
-  if (in == "october"   || in == "oct")     return 10;
-  if (in == "november"  || in == "nov")     return 11;
-  if (in == "december"  || in == "dec")     return 12;
+  if (in == STRING_DATE_JANUARY_LONG   || in == STRING_DATE_JANUARY_SHORT  )     return  1;
+  if (in == STRING_DATE_FEBRUARY_LONG  || in == STRING_DATE_FEBRUARY_SHORT )     return  2;
+  if (in == STRING_DATE_MARCH_LONG     || in == STRING_DATE_MARCH_SHORT    )     return  3;
+  if (in == STRING_DATE_APRIL_LONG     || in == STRING_DATE_APRIL_SHORT    )     return  4;
+  if (in == STRING_DATE_MAY_LONG       || in == STRING_DATE_MAY_SHORT      )     return  5;
+  if (in == STRING_DATE_JUNE_LONG      || in == STRING_DATE_JUNE_SHORT     )     return  6;
+  if (in == STRING_DATE_JULY_LONG      || in == STRING_DATE_JULY_SHORT     )     return  7;
+  if (in == STRING_DATE_AUGUST_LONG    || in == STRING_DATE_AUGUST_SHORT   )     return  8;
+  if (in == STRING_DATE_SEPTEMBER_LONG || in == STRING_DATE_SEPTEMBER_SHORT)     return  9;
+  if (in == STRING_DATE_OCTOBER_LONG   || in == STRING_DATE_OCTOBER_SHORT  )     return 10;
+  if (in == STRING_DATE_NOVEMBER_LONG  || in == STRING_DATE_NOVEMBER_SHORT )     return 11;
+  if (in == STRING_DATE_DECEMBER_LONG  || in == STRING_DATE_DECEMBER_SHORT )     return 12;
 
   return -1;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+int Date::length (const std::string& format)
+{
+  int total = 0;
+
+  std::string::const_iterator i;
+  for (i = format.begin (); i != format.end (); ++i)
+  {
+    switch (*i)
+    {
+    case 'm':
+    case 'M':
+    case 'd':
+    case 'D':
+    case 'y':
+    case 'A':
+    case 'b':
+    case 'B':
+    case 'V':
+    case 'h':
+    case 'H':
+    case 'N':
+    case 'S': total += 2; break;
+    case 'a': total += 3; break;
+    case 'Y': total += 4; break;
+    default:  total += 1; break;
+    }
+  }
+
+  return total;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 time_t Date::easter (int year)
 {
@@ -716,83 +562,83 @@ time_t Date::easter (int year)
 ////////////////////////////////////////////////////////////////////////////////
 int Date::month () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_mon + 1;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Date::day () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_mday;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Date::year () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_year + 1900;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Date::hour () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_hour;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Date::minute () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_min;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 int Date::second () const
 {
-  struct tm* t = localtime (&mT);
+  struct tm* t = localtime (&_t);
   return t->tm_sec;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::operator== (const Date& rhs)
+bool Date::operator== (const Date& rhs) const
 {
-  return rhs.mT == mT;
+  return rhs._t == _t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::operator!= (const Date& rhs)
+bool Date::operator!= (const Date& rhs) const
 {
-  return rhs.mT != mT;
+  return rhs._t != _t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::operator<  (const Date& rhs)
+bool Date::operator<  (const Date& rhs) const
 {
-  return mT < rhs.mT;
+  return _t < rhs._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::operator>  (const Date& rhs)
+bool Date::operator>  (const Date& rhs) const
 {
-  return mT > rhs.mT;
+  return _t > rhs._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::operator<= (const Date& rhs)
+bool Date::operator<= (const Date& rhs) const
 {
-  return mT <= rhs.mT;
+  return _t <= rhs._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::operator>= (const Date& rhs)
+bool Date::operator>= (const Date& rhs) const
 {
-  return mT >= rhs.mT;
+  return _t >= rhs._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::sameHour (const Date& rhs)
+bool Date::sameHour (const Date& rhs) const
 {
   if (this->year ()  == rhs.year ()  &&
       this->month () == rhs.month () &&
@@ -804,7 +650,7 @@ bool Date::sameHour (const Date& rhs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::sameDay (const Date& rhs)
+bool Date::sameDay (const Date& rhs) const
 {
   if (this->year ()  == rhs.year ()  &&
       this->month () == rhs.month () &&
@@ -815,7 +661,7 @@ bool Date::sameDay (const Date& rhs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::sameMonth (const Date& rhs)
+bool Date::sameMonth (const Date& rhs) const
 {
   if (this->year ()  == rhs.year () &&
       this->month () == rhs.month ())
@@ -825,7 +671,7 @@ bool Date::sameMonth (const Date& rhs)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Date::sameYear (const Date& rhs)
+bool Date::sameYear (const Date& rhs) const
 {
   if (this->year () == rhs.year ())
     return true;
@@ -836,33 +682,33 @@ bool Date::sameYear (const Date& rhs)
 ////////////////////////////////////////////////////////////////////////////////
 Date Date::operator- (const int delta)
 {
-  return Date (mT - delta);
+  return Date (_t - delta);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Date Date::operator+ (const int delta)
 {
-  return Date (mT + delta);
+  return Date (_t + delta);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Date& Date::operator+= (const int delta)
 {
-  mT += (time_t) delta;
+  _t += (time_t) delta;
   return *this;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 Date& Date::operator-= (const int delta)
 {
-  mT -= (time_t) delta;
+  _t -= (time_t) delta;
   return *this;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 time_t Date::operator- (const Date& rhs)
 {
-  return mT - rhs.mT;
+  return _t - rhs._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -876,7 +722,7 @@ void Date::operator-- ()
                     hour (),
                     minute (),
                     second ());
-  mT = yesterday.mT;
+  _t = yesterday._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -890,7 +736,7 @@ void Date::operator-- (int)
                     hour (),
                     minute (),
                     second ());
-  mT = yesterday.mT;
+  _t = yesterday._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -904,7 +750,7 @@ void Date::operator++ ()
                    hour (),
                    minute (),
                    second ());
-  mT = tomorrow.mT;
+  _t = tomorrow._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -918,17 +764,16 @@ void Date::operator++ (int)
                    hour (),
                    minute (),
                    second ());
-  mT = tomorrow.mT;
+  _t = tomorrow._t;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 bool Date::isEpoch (const std::string& input)
 {
-  if (digitsOnly (input)    &&
-      input.length () >   8 &&
+  if (digitsOnly (input) &&
       input.length () <= 10 )
   {
-    mT = (time_t) atoi (input.c_str ());
+    _t = (time_t) atoi (input.c_str ());
     return true;
   }
 
@@ -936,7 +781,7 @@ bool Date::isEpoch (const std::string& input)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// If the input string looks like a relative date, determine that date, set mT
+// If the input string looks like a relative date, determine that date, set _t
 // and return true.
 //
 // What is a relative date?  All of the following should be recognizable, and
@@ -957,39 +802,12 @@ bool Date::isRelativeDate (const std::string& input)
   Date today;
 
   std::vector <std::string> supported;
-  supported.push_back ("monday");
-  supported.push_back ("tuesday");
-  supported.push_back ("wednesday");
-  supported.push_back ("thursday");
-  supported.push_back ("friday");
-  supported.push_back ("saturday");
-  supported.push_back ("sunday");
-  supported.push_back ("today");
-  supported.push_back ("tomorrow");
-  supported.push_back ("yesterday");
-  supported.push_back ("eow");
-  supported.push_back ("eoww");
-  supported.push_back ("eocw");
-  supported.push_back ("eom");
-  supported.push_back ("eoy");
-  supported.push_back ("sow");
-  supported.push_back ("soww");
-  supported.push_back ("socw");
-  supported.push_back ("som");
-  supported.push_back ("soy");
-  supported.push_back ("goodfriday");
-  supported.push_back ("easter");
-  supported.push_back ("eastermonday");
-  supported.push_back ("ascension");
-  supported.push_back ("pentecost");
-  supported.push_back ("midsommar");
-  supported.push_back ("midsommarafton");
-  supported.push_back ("now");
-  supported.push_back ("later");
-  supported.push_back ("someday");
+  for (unsigned int i = 0; i < NUM_RELATIVES; ++i)
+    supported.push_back (relatives[i]);
 
+  // Hard-coded 3, despite rc.abbreviation.minimum.
   std::vector <std::string> matches;
-  if (autoComplete (in, supported, matches) == 1)
+  if (autoComplete (in, supported, matches, 3) == 1)
   {
     std::string found = matches[0];
 
@@ -1024,7 +842,7 @@ bool Date::isRelativeDate (const std::string& input)
       today.toMDY (m, d, y);
       Date then (m, d, y);
 
-      mT = then.mT;
+      _t = then._t;
       return true;
     }
     else if (found == "today")
@@ -1032,7 +850,7 @@ bool Date::isRelativeDate (const std::string& input)
       Date then (today.month (),
                  today.day (),
                  today.year ());
-      mT = then.mT;
+      _t = then._t;
       return true;
     }
     else if (found == "tomorrow")
@@ -1040,7 +858,7 @@ bool Date::isRelativeDate (const std::string& input)
       Date then (today.month (),
                  today.day (),
                  today.year ());
-      mT = then.mT + 86400;
+      _t = then._t + 86400;
       return true;
     }
     else if (found == "yesterday")
@@ -1048,7 +866,7 @@ bool Date::isRelativeDate (const std::string& input)
       Date then (today.month (),
                  today.day (),
                  today.year ());
-      mT = then.mT - 86400;
+      _t = then._t - 86400;
       return true;
     }
     else if (found == "eom")
@@ -1056,13 +874,22 @@ bool Date::isRelativeDate (const std::string& input)
       Date then (today.month (),
                  daysInMonth (today.month (), today.year ()),
                  today.year ());
-      mT = then.mT;
+      _t = then._t;
+      return true;
+    }
+    else if (found == "eoq")
+    {
+      int eoq_month = today.month () + 2 - (today.month () - 1) % 3;
+      Date then (eoq_month,
+                 daysInMonth (eoq_month, today.year ()),
+                 today.year ());
+      _t = then._t;
       return true;
     }
     else if (found == "eoy")
     {
       Date then (12, 31, today.year ());
-      mT = then.mT;
+      _t = then._t;
       return true;
     }
     else if (found == "som")
@@ -1075,43 +902,56 @@ bool Date::isRelativeDate (const std::string& input)
         y++;
       }
       Date then (m, 1, y);
-      mT = then.mT;
+      _t = then._t;
+      return true;
+    }
+    else if (found == "soq")
+    {
+      int m = today.month () + 3 - (today.month () - 1) % 3;
+      int y = today.year ();
+      if (m > 12)
+      {
+        m -=12;
+        y++;
+      }
+      Date then (m , 1, y);
+      _t = then._t;
       return true;
     }
     else if (found == "soy")
     {
       Date then (1, 1, today.year () + 1);
-      mT = then.mT;
+      _t = then._t;
       return true;
     }
     else if (found == "goodfriday")
     {
       Date then (Date::easter(today.year()));
-      mT = then.mT - 86400*2;
+      _t = then._t - 86400*2;
       return true;
     }
     else if (found == "easter")
     {
       Date then (Date::easter(today.year()));
-      mT = then.mT;
+      _t = then._t;
       return true;
     }
     else if (found == "eastermonday")
     {
       Date then (Date::easter(today.year()));
-      mT = then.mT + 86400;
+      _t = then._t + 86400;
       return true;
     }
     else if (found == "ascension")
     {
       Date then (Date::easter(today.year()));
-      mT = then.mT + 86400*39;
+      _t = then._t + 86400*39;
       return true;
     }
     else if (found == "pentecost")
     {
       Date then (Date::easter(today.year()));
-      mT = then.mT + 86400*49;
+      _t = then._t + 86400*49;
       return true;
     }
     else if (found == "midsommar")
@@ -1121,7 +961,7 @@ bool Date::isRelativeDate (const std::string& input)
         Date then (6, midsommar, today.year ());
         if (6 == then.dayOfWeek ())
         {
-          mT = then.mT;
+          _t = then._t;
           return true;
         }
       }
@@ -1133,20 +973,20 @@ bool Date::isRelativeDate (const std::string& input)
         Date then (6, midsommar, today.year ());
         if (5 == then.dayOfWeek ())
         {
-          mT = then.mT;
+          _t = then._t;
           return true;
         }
       }
     }
     else if (found == "now")
     {
-      mT = time (NULL);
+      _t = time (NULL);
       return true;
     }
     else if (found == "later" || found == "someday")
     {
       Date then (1, 18, 2038);
-      mT = then.mT;
+      _t = then._t;
       return true;
     }
   }
@@ -1186,7 +1026,7 @@ bool Date::isRelativeDate (const std::string& input)
             number <= Date::daysInMonth (m, y))
         {
           Date then (m, number, y);
-          mT = then.mT;
+          _t = then._t;
           return true;
         }
 
@@ -1203,7 +1043,7 @@ bool Date::isRelativeDate (const std::string& input)
         while (number > Date::daysInMonth (m, y));
 
         Date then (m, number, y);
-        mT = then.mT;
+        _t = then._t;
         return true;
       }
     }
@@ -1213,3 +1053,14 @@ bool Date::isRelativeDate (const std::string& input)
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+const std::vector <std::string> Date::get_relatives ()
+{
+  std::vector <std::string> all;
+  for (unsigned int i = 0; i < NUM_RELATIVES; ++i)
+    if (strcmp (relatives[i], "-"))
+      all.push_back (relatives[i]);
+
+  return all;
+}
+
+////////////////////////////////////////////////////////////////////////////////
