@@ -135,11 +135,16 @@ Duration::Duration (const std::string& input)
 {
   if (digitsOnly (input))
   {
-    _secs = (time_t) strtol (input.c_str (), NULL, 10);
-    _negative = false;
+    time_t value = (time_t) strtol (input.c_str (), NULL, 10);
+    if (value == 0 || value > 60)
+    {
+      _secs = value;
+      _negative = false;
+      return;
+    }
   }
-  else
-    parse (input);
+
+  parse (input);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -236,8 +241,8 @@ std::string Duration::format () const
     sprintf (formatted, "%s%.1f yrs", (_negative ? "-" : ""), (days / 365));
   else if (_secs > 86400 * 84)
     sprintf (formatted, "%s%1d mth%s",
-                        (_negative ? "-" : ""),  (int) (float) (days / 30.6),
-                        ((int) (float) (days / 30.6) == 1 ? "" : "s"));
+                        (_negative ? "-" : ""),  (int) (float) (days / 30),
+                        ((int) (float) (days / 30) == 1 ? "" : "s"));
   else if (_secs > 86400 * 13)
     sprintf (formatted, "%s%d wk%s",
                         (_negative ? "-" : ""),  (int) (float) (days / 7.0),
@@ -270,12 +275,12 @@ std::string Duration::formatCompact () const
   char formatted[24];
   float days = (float) _secs / 86400.0;
 
-       if (_secs >= 86400 * 365) sprintf (formatted, "%s%.1fy", (_negative ? "-" : ""), (days / 365));
-  else if (_secs >= 86400 * 84)  sprintf (formatted, "%s%1dmo", (_negative ? "-" : ""), (int) (float) (days / 30.6));
+       if (_secs >= 86400 * 365) sprintf (formatted, "%s%.1fy", (_negative ? "-" : ""), (days / 365.0));
+  else if (_secs >= 86400 * 84)  sprintf (formatted, "%s%1dmo", (_negative ? "-" : ""), (int) (days / 30));
   else if (_secs >= 86400 * 13)  sprintf (formatted, "%s%dwk",  (_negative ? "-" : ""), (int) (float) (days / 7.0));
   else if (_secs >= 86400)       sprintf (formatted, "%s%dd",   (_negative ? "-" : ""), (int) days);
-  else if (_secs >= 3600)        sprintf (formatted, "%s%dh",   (_negative ? "-" : ""), (int) (float) (_secs / 3600));
-  else if (_secs >= 60)          sprintf (formatted, "%s%dm",   (_negative ? "-" : ""), (int) (float) (_secs / 60));
+  else if (_secs >= 3600)        sprintf (formatted, "%s%dh",   (_negative ? "-" : ""), (int) (_secs / 3600));
+  else if (_secs >= 60)          sprintf (formatted, "%s%dm",   (_negative ? "-" : ""), (int) (_secs / 60));
   else if (_secs >= 1)           sprintf (formatted, "%s%ds",   (_negative ? "-" : ""), (int) _secs);
   else                           strcpy (formatted, "-");
 
@@ -298,6 +303,15 @@ std::string Duration::formatPrecise () const
   return std::string (formatted);
 }
 
+
+////////////////////////////////////////////////////////////////////////////////
+std::string Duration::formatSeconds () const
+{
+  char formatted[24];
+  sprintf (formatted, "%s%ldsec", (_negative ? "-" : ""), _secs);
+  return std::string (formatted);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 bool Duration::operator< (const Duration& other)
 {
@@ -362,8 +376,9 @@ bool Duration::valid (const std::string& input)
   n.getUntilEOS (units);
 
   // Non-trivial value with no units means the duration is specified in
-  // seconds, and therefore a time_t.  Consider it valid.
-  if (value != 0.0 &&
+  // seconds, and therefore a time_t.  Consider it valid provided it is >= 60.
+  if (value != 0.0  &&
+      value >= 60.0 &&
       units == "")
     return true;
 
@@ -401,13 +416,13 @@ void Duration::parse (const std::string& input)
     _negative = false;
 
   // If no units are provided, assume seconds.
-  if (n.depleted ())
+  std::string units;
+  if (n.depleted () && value >= 60)
   {
     _secs = (long) value;
     return;
   }
 
-  std::string units;
   n.getUntilEOS (units);
 
   // Auto complete against all supported durations.
