@@ -29,6 +29,7 @@
 #define INCLUDED_PATH
 #define L10N                                           // Localization complete.
 
+#include <iostream>
 #include <vector>
 #include <string>
 
@@ -62,5 +63,7 @@ public:
   std::string _data;
 };
 
+std::ostream& operator<< (std::ostream&, const Path&);
+
 #endif
 ////////////////////////////////////////////////////////////////////////////////
