@@ -51,9 +51,9 @@ extern Context context;
 #define DEBUG_OUTPUT 0
 
 #if DEBUG_OUTPUT > 0
-  #define DEBUG_STR(str)       std::cout << "DEBUG: " << str << "\n"; std::cout.flush()
-  #define DEBUG_STR_PART(str)  std::cout << "DEBUG: " << str; std::cout.flush()
-  #define DEBUG_STR_END(str)   std::cout << str << "\n"; std::cout.flush()
+  #define DEBUG_STR(str)       std::cerr << "DEBUG: " << str << "\n"; std::cerr.flush()
+  #define DEBUG_STR_PART(str)  std::cerr << "DEBUG: " << str; std::cerr.flush()
+  #define DEBUG_STR_END(str)   std::cerr << str << "\n"; std::cerr.flush()
 #else
   #define DEBUG_STR(str)
   #define DEBUG_STR_PART(str)
@@ -66,9 +66,8 @@ TF2::TF2 ()
 , _dirty (false)
 , _loaded_tasks (false)
 , _loaded_lines (false)
-, _loaded_contents (false)
 , _has_ids (false)
-, _contents ("")
+, _auto_dep_scan (false)
 {
 }
 
@@ -107,12 +106,43 @@ const std::vector <std::string>& TF2::get_lines ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-const std::string& TF2::get_contents ()
+// Locate task by id.
+bool TF2::get (int id, Task& task)
 {
-  if (! _loaded_contents)
-    load_contents ();
+  if (! _loaded_tasks)
+    load_tasks ();
 
-  return _contents;
+  std::vector <Task>::iterator i;
+  for (i = _tasks.begin (); i != _tasks.end (); ++i)
+  {
+    if (i->id == id)
+    {
+      task = *i;
+      return true;
+    }
+  }
+
+  return false;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Locate task by uuid.
+bool TF2::get (const std::string& uuid, Task& task)
+{
+  if (! _loaded_tasks)
+    load_tasks ();
+
+  std::vector <Task>::iterator i;
+  for (i = _tasks.begin (); i != _tasks.end (); ++i)
+  {
+    if (i->get ("uuid") == uuid)
+    {
+      task = *i;
+      return true;
+    }
+  }
+
+  return false;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -282,6 +312,9 @@ void TF2::load_tasks ()
         _U2I[task.get ("uuid")] = task.id;
       }
     }
+ 
+    if (_auto_dep_scan)
+      dependency_scan ();
 
     _loaded_tasks = true;
   }
@@ -297,25 +330,13 @@ void TF2::load_tasks ()
 ////////////////////////////////////////////////////////////////////////////////
 void TF2::load_lines ()
 {
-  if (! _loaded_contents)
-    load_contents ();
-
-  split_minimal (_lines, _contents, '\n');
-  _loaded_lines = true;
-}
-
-////////////////////////////////////////////////////////////////////////////////
-void TF2::load_contents ()
-{
-  _contents = "";
-
   if (_file.open ())
   {
     if (context.config.getBoolean ("locking"))
       _file.lock ();
 
-    _file.read (_contents);
-    _loaded_contents = true;
+    _file.read (_lines);
+    _loaded_lines = true;
   }
 }
 
@@ -365,6 +386,12 @@ void TF2::has_ids ()
   _has_ids = true;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+void TF2::auto_dep_scan ()
+{
+  _auto_dep_scan = true;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Completely wipe it all clean.
 void TF2::clear ()
@@ -373,13 +400,11 @@ void TF2::clear ()
   _dirty           = false;
   _loaded_tasks    = false;
   _loaded_lines    = false;
-  _loaded_contents = false;
 
-  _contents        = "";
-
-  // Note that the actual file name, and _has_ids are deliberately not cleared.
+  // Note that these are deliberately not cleared.
   //_file._data      = "";
   //_has_ids         = false;
+  //_auto_dep_scan   = false;
 
   _tasks.clear ();
   _added_tasks.clear ();
@@ -390,6 +415,49 @@ void TF2::clear ()
   _U2I.clear ();
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// For any task that has depenencies, follow the chain of dependencies until the
+// end.  Along the way, update the Task::is_blocked and Task::is_blocking data
+// cache.
+void TF2::dependency_scan ()
+{
+  // Iterate and modify TDB2 in-place.  Don't do this at home.
+  std::vector <Task>::iterator left;
+  for (left  = _tasks.begin ();
+       left != _tasks.end ();
+       ++left)
+  {
+    if (left->has ("depends"))
+    {
+      std::vector <std::string> deps;
+      left->getDependencies (deps);
+
+      std::vector <std::string>::iterator d;
+      for (d = deps.begin (); d != deps.end (); ++d)
+      {
+        std::vector <Task>::iterator right;
+        for (right  = _tasks.begin ();
+             right != _tasks.end ();
+             ++right)
+        {
+          if (right->get ("uuid") == *d)
+          {
+            Task::status status = right->getStatus ();
+            if (status != Task::completed &&
+                status != Task::deleted)
+            {
+              left->is_blocked = true;
+              right->is_blocking = true;
+            }
+
+            break;
+          }
+        }
+      }
+    }
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 const std::string TF2::dump ()
 {
@@ -418,10 +486,9 @@ const std::string TF2::dump ()
   std::string tasks_modified = yellow.colorize (rightJustifyZero ((int) _modified_tasks.size (), 3));
   std::string lines          = green.colorize  (rightJustifyZero ((int) _lines.size (),          4));
   std::string lines_added    = red.colorize    (rightJustifyZero ((int) _added_lines.size (),    3));
-  std::string contents       = green.colorize  (rightJustifyZero ((int) _contents.size (),       6));
 
   char buffer[256];  // Composed string is actually 246 bytes.  Yikes.
-  snprintf (buffer, 256, "%14s %s %s T%s+%s~%s L%s+%s C%s",
+  snprintf (buffer, 256, "%14s %s %s T%s+%s~%s L%s+%s",
             label.c_str (),
             mode.c_str (),
             hygiene.c_str (),
@@ -429,8 +496,7 @@ const std::string TF2::dump ()
             tasks_added.c_str (),
             tasks_modified.c_str (),
             lines.c_str (),
-            lines_added.c_str (),
-            contents.c_str ());
+            lines_added.c_str ());
 
   return std::string (buffer);
 }
@@ -442,6 +508,10 @@ TDB2::TDB2 ()
 {
   // Mark the pending file as the only one that has ID numbers.
   pending.has_ids ();
+
+  // Indicate that dependencies should be automatically scanned on startup,
+  // setting Task::is_blocked and Task::is_blocking accordingly.
+  pending.auto_dep_scan ();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -505,7 +575,7 @@ void TDB2::add (Task& task)
 void TDB2::modify (Task& task)
 {
   // Ensure the task is consistent, and provide defaults if necessary.
-  task.validate ();
+  task.validate (false);
 
   // Find task, overwrite it.
   Task original;
@@ -716,8 +786,6 @@ void TDB2::merge (const std::string& mergeFile)
     }
   }
 
-  std::cout << "\n";
-
   ///////////////////////////////////////
   // branch-off point found:
   if (found)
@@ -940,10 +1008,10 @@ void TDB2::merge (const std::string& mergeFile)
   }
   else if (rit == r.end ())
   {
-    // nothing happend on the remote branch
+    // nothing happened on the remote branch
     // local branch is up-to-date
 
-    // nothing happend on the local branch either
+    // nothing happened on the local branch either
 
     // break, to suppress autopush
     if (lit == l.end ())
@@ -1498,8 +1566,8 @@ void TDB2::revert ()
 ////////////////////////////////////////////////////////////////////////////////
 // Scans the pending tasks for any that are completed or deleted, and if so,
 // moves them to the completed.data file.  Returns a count of tasks moved.
-// Now reverts expired waiting tasks to pending.
-// Now cleans up dangling dependencies.
+// Reverts expired waiting tasks to pending.
+// Cleans up dangling dependencies.
 //
 // Possible scenarios:
 // - task in pending that needs to be in completed
@@ -1536,13 +1604,14 @@ int TDB2::gc ()
          ++task)
     {
       status = task->get ("status");
-      if (status == "pending" || status == "recurring")
+      if (status == "pending" ||
+          status == "recurring")
       {
         pending_tasks_after.push_back (*task);
       }
       else if (status == "waiting")
       {
-        Date wait (task->get ("wait"));
+        Date wait (task->get_date ("wait"));
         if (wait < now)
         {
           task->set ("status", "pending");
@@ -1568,12 +1637,26 @@ int TDB2::gc ()
     {
       status = task->get ("status");
       if (status == "pending" ||
-          status == "waiting")
+          status == "recurring")
       {
         pending_tasks_after.push_back (*task);
         pending_changes = true;
         completed_changes = true;
       }
+      else if (status == "waiting")
+      {
+        Date wait (task->get_date ("wait"));
+        if (wait < now)
+        {
+          task->set ("status", "pending");
+          task->remove ("wait");
+          pending_tasks_after.push_back (*task);
+          pending_changes = true;
+          completed_changes = true;
+        }
+
+        pending_tasks_after.push_back (*task);
+      }
       else
       {
         completed_tasks_after.push_back (*task);
@@ -1583,17 +1666,16 @@ int TDB2::gc ()
     // Only recreate the pending.data file if necessary.
     if (pending_changes)
     {
-      pending.clear ();
+      pending._tasks = pending_tasks_after;
       pending._dirty = true;
       pending._loaded_tasks = true;
       _id = 1;
 
-      for (task = pending_tasks_after.begin ();
-           task != pending_tasks_after.end ();
+      for (task = pending._tasks.begin ();
+           task != pending._tasks.end ();
            ++task)
       {
         task->id = _id++;
-        pending._tasks.push_back (*task);
       }
 
       // Note: deliberately no commit.
@@ -1602,15 +1684,10 @@ int TDB2::gc ()
     // Only recreate the completed.data file if necessary.
     if (completed_changes)
     {
-      completed.clear ();
+      completed._tasks = completed_tasks_after;
       completed._dirty = true;
       completed._loaded_tasks = true;
 
-      for (task = completed_tasks_after.begin ();
-           task != completed_tasks_after.end ();
-           ++task)
-        completed._tasks.push_back (*task);
-
       // Note: deliberately no commit.
     }
 
@@ -1653,75 +1730,19 @@ const std::vector <Task> TDB2::all_tasks ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Locate task by ID.
+// Locate task by ID, wherever it is.
 bool TDB2::get (int id, Task& task)
 {
-  // First load and scan pending.
-  if (! pending._loaded_tasks)
-    pending.load_tasks ();
-
-  std::vector <Task>::iterator i;
-  for (i = pending._tasks.begin (); i != pending._tasks.end (); ++i)
-  {
-    if (i->id == id)
-    {
-      task = *i;
-      return true;
-    }
-  }
-
-  // Next load and scan completed.
-  // Note that this is harmless, because it is only performed if the above
-  // load and search fails.
-  if (! completed._loaded_tasks)
-    completed.load_tasks ();
-
-  for (i = completed._tasks.begin (); i != completed._tasks.end (); ++i)
-  {
-    if (i->id == id)
-    {
-      task = *i;
-      return true;
-    }
-  }
-
-  return false;
+  return pending.get   (id, task) ||
+         completed.get (id, task);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-// Locate task by UUID.
+// Locate task by UUID, wherever it is.
 bool TDB2::get (const std::string& uuid, Task& task)
 {
-  // First load and scan pending.
-  if (! pending._loaded_tasks)
-    pending.load_tasks ();
-
-  std::vector <Task>::iterator i;
-  for (i = pending._tasks.begin (); i != pending._tasks.end (); ++i)
-  {
-    if (i->get ("uuid") == uuid)
-    {
-      task = *i;
-      return true;
-    }
-  }
-
-  // Next load and scan completed.
-  // Note that this is harmless, because it is only performed if the above
-  // load and search fails.
-  if (! completed._loaded_tasks)
-    completed.load_tasks ();
-
-  for (i = completed._tasks.begin (); i != completed._tasks.end (); ++i)
-  {
-    if (i->get ("uuid") == uuid)
-    {
-      task = *i;
-      return true;
-    }
-  }
-
-  return false;
+  return pending.get   (uuid, task) ||
+         completed.get (uuid, task);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
