@@ -85,6 +85,7 @@ int CmdShow::execute (std::string& output)
     " color.active"
     " color.alternate"
     " color.blocked"
+    " color.blocking"
     " color.burndown.done"
     " color.burndown.pending"
     " color.burndown.started"
@@ -100,6 +101,7 @@ int CmdShow::execute (std::string& output)
     " color.deleted"
     " color.due"
     " color.due.today"
+    " color.error"
     " color.footnote"
     " color.header"
     " color.history.add"
@@ -112,6 +114,7 @@ int CmdShow::execute (std::string& output)
     " color.pri.M"
     " color.pri.none"
     " color.recurring"
+    " color.scheduled"
     " color.summary.background"
     " color.summary.bar"
     " color.sync.added"
@@ -127,7 +130,9 @@ int CmdShow::execute (std::string& output)
     " data.location"
     " dateformat"
     " dateformat.annotation"
+    " dateformat.edit"
     " dateformat.holiday"
+    " dateformat.info"
     " dateformat.report"
     " debug"
     " default.command"
@@ -185,6 +190,7 @@ int CmdShow::execute (std::string& output)
     " taskd.credentials"
     " undo.style"
     " urgency.active.coefficient"
+    " urgency.scheduled.coefficient"
     " urgency.annotations.coefficient"
     " urgency.blocked.coefficient"
     " urgency.blocking.coefficient"
@@ -232,7 +238,8 @@ int CmdShow::execute (std::string& output)
           i->substr (0,  6) != "merge."                &&
           i->substr (0,  4) != "uda."                  &&
           i->substr (0, 21) != "urgency.user.project." &&
-          i->substr (0, 17) != "urgency.user.tag.")
+          i->substr (0, 17) != "urgency.user.tag."     &&
+          i->substr (0, 12) != "urgency.uda.")
       {
         unrecognized.push_back (*i);
       }
@@ -372,15 +379,6 @@ int CmdShow::execute (std::string& output)
   }
 #endif
 
-  // Check for bad values in rc.annotations.
-  // TODO Deprecated.
-  std::string annotations = context.config.get ("annotations");
-  if (annotations != "full"   &&
-      annotations != "sparse" &&
-      annotations != "none")
-    out << format (STRING_CMD_SHOW_CONFIG_ERROR, "annotations", annotations)
-        << "\n";
-
   // Check for bad values in rc.calendar.details.
   std::string calendardetails = context.config.get ("calendar.details");
   if (calendardetails != "full"   &&
