@@ -47,6 +47,7 @@ Date getNextRecurrence (Date&, std::string&);
 bool generateDueDates (Task&, std::vector <Date>&);
 void updateRecurrenceMask (Task&);
 int getDueState (const std::string&);
+int getDueState (const Date&);
 bool nag (Task&);
 
 // rules.cpp
@@ -54,12 +55,11 @@ void initializeColorRules ();
 void autoColorize (Task&, Color&);
 std::string colorizeHeader (const std::string&);
 std::string colorizeFootnote (const std::string&);
+std::string colorizeError (const std::string&);
 std::string colorizeDebug (const std::string&);
 
 // dependency.cpp
-bool dependencyIsBlocked (const Task&);
 void dependencyGetBlocked (const Task&, std::vector <Task>&);
-bool dependencyIsBlocking (const Task&);
 void dependencyGetBlocking (const Task&, std::vector <Task>&);
 bool dependencyIsCircular (const Task&);
 void dependencyChainOnComplete (Task&);
@@ -69,8 +69,8 @@ void dependencyChainOnModify (Task&, Task&);
 // feedback.cpp
 bool taskDiff (const Task&, const Task&);
 std::string taskDifferences (const Task&, const Task&);
-std::string taskInfoDifferences (const Task&, const Task&);
-std::string renderAttribute (const std::string&, const std::string&);
+std::string taskInfoDifferences (const Task&, const Task&, const std::string&);
+std::string renderAttribute (const std::string&, const std::string&, const std::string& format = "");
 void feedback_affected (const std::string&);
 void feedback_affected (const std::string&, int);
 void feedback_affected (const std::string&, const Task&);
@@ -78,6 +78,7 @@ void feedback_special_tags (const Task&, const std::string&);
 void feedback_unblocked (const Task&);
 std::string onProjectChange (Task&, bool scope = true);
 std::string onProjectChange (Task&, Task&);
+std::string onExpiration (Task&);
 
 // sort.cpp
 void sort_tasks (std::vector <Task>&, std::vector <int>&, const std::string&);
