@@ -77,7 +77,16 @@ const std::string escape (const std::string&, char);
 const std::vector<std::string> indentTree (
   const std::vector<std::string>&,
   const std::string& whitespace = "  ",
-  char delimiter='.');
+  char delimiter = '.');
+
+const std::string indentProject (
+  const std::string&,
+  const std::string& whitespace = "  ",
+  char delimiter = '.');
+
+#ifndef HAVE_TIMEGM
+  time_t timegm (struct tm *tm);
+#endif
 
 #endif
 ////////////////////////////////////////////////////////////////////////////////
